@@ -498,7 +498,6 @@ class RuleExhaustiveBallot(Rule):
     def w_(self):
         self.mylog("Compute w", 1)
         plurality_elimination_engine = self.profile_.plurality_elimination_engine()
-        loser = None
         for r in range(self.profile_.n_c - 1):
             if r != 0:
                 plurality_elimination_engine.update_scores()
@@ -1264,6 +1263,15 @@ class RuleExhaustiveBallot(Rule):
         self._example_path_um = {c: None for c in range(self.profile_.n_c)}
         return True
 
+    def _um_preliminary_checks_c_(self, c):
+        if self.um_option not in {'fast', 'lazy'} or self.cm_option not in {'fast', 'lazy'}:
+            if (
+                self.w_ == self.profile_.condorcet_winner_rk_ctb
+                and not self.profile_.c_might_be_there_when_cw_is_eliminated_irv_style[c]
+            ):
+                # Impossible to manipulate with n_m manipulators
+                self._candidates_um[c] = False
+
     def _um_main_work_c_(self, c):
         """
             >>> profile = Profile(preferences_rk=[
@@ -1371,15 +1379,13 @@ class RuleExhaustiveBallot(Rule):
             self.mylogv('CM: Preliminary checks: Update _example_path_cm[c] = _example_path_TM[c] =',
                         self.example_path_tm_c_(c), 3)
             self._example_path_cm[c] = self.example_path_tm_c_(c)
-        if self.cm_option != 'fast':
-            if (
-                self.w_ == self.profile_.condorcet_winner_rk_ctb
-                and not self.profile_.c_might_be_there_when_cw_is_eliminated_irv_style[c]
-            ):
-                # Impossible to manipulate with n_m manipulators
-                n_m = self.profile_.matrix_duels_ut[c, self.w_]
-                self._update_necessary(self._necessary_coalition_size_cm, c, n_m + 1,
-                                       'CM: Update necessary_coalition_size_cm[c] = n_m + 1 =')
+        if self.cm_option not in {'fast', 'lazy'}:
+            if self.w_ == self.profile_.condorcet_winner_rk_ctb:
+                self._update_necessary(
+                    self._necessary_coalition_size_cm, c,
+                    self.profile_.necessary_coalition_size_to_break_irv_immunity[c],
+                    'CM: Preliminary checks: IRV-Immunity => \n    necessary_coalition_size_cm[c] = '
+                )
 
     def _cm_main_work_c_(self, c, optimize_bounds):
         """
