@@ -328,6 +328,17 @@ class RuleIRVAverage(Rule):
     def meets_majority_favorite_c_rk_ctb(self):
         return True
 
+    # %% Unison manipulation (UM)
+
+    def _um_preliminary_checks_c_(self, c):
+        if self.um_option not in {'fast', 'lazy'} or self.cm_option not in {'fast', 'lazy'}:
+            if (
+                self.w_ == self.profile_.condorcet_winner_rk_ctb
+                and not self.profile_.c_might_be_there_when_cw_is_eliminated_irv_style[c]
+            ):
+                # Impossible to manipulate with n_m manipulators
+                self._candidates_um[c] = False
+
     # %% Coalition Manipulation (CM)
 
     def _cm_preliminary_checks_c_subclass_(self, c, optimize_bounds):
@@ -337,15 +348,13 @@ class RuleIRVAverage(Rule):
         least if utility preferences are strict). Then at some point, `w` must have a plurality score under average, at
         a moment where `c` is still present in the election.
         """
-        if self.cm_option != 'fast':
-            if (
-                self.w_ == self.profile_.condorcet_winner_rk_ctb
-                and not self.profile_.c_might_be_there_when_cw_is_eliminated_irv_style[c]
-            ):
-                # Impossible to manipulate with n_m manipulators
-                n_m = self.profile_.matrix_duels_ut[c, self.w_]
-                self._update_necessary(self._necessary_coalition_size_cm, c, n_m + 1,
-                                       'CM: Update necessary_coalition_size_cm[c] = n_m + 1 =')
+        if self.cm_option not in {'fast', 'lazy'}:
+            if self.w_ == self.profile_.condorcet_winner_rk_ctb:
+                self._update_necessary(
+                    self._necessary_coalition_size_cm, c,
+                    self.profile_.necessary_coalition_size_to_break_irv_immunity[c],
+                    'CM: Preliminary checks: IRV-Immunity => \n    necessary_coalition_size_cm[c] = '
+                )
 
     def _cm_aux_(self, c, ballots_m, preferences_rk_s):
         profile_test = Profile(preferences_rk=np.concatenate((preferences_rk_s, ballots_m)), sort_voters=False)
